module Main exposing (..)

import Browser
import Html exposing (Html, button, div, text)
import Html.Events exposing (onClick)
import Http
import Json.Decode exposing (Decoder, map6, field, int, string, list, nullable)


type alias Model =
    { presidents : Maybe (List President) }

type alias President =
 {
  pres_name:String,
  birth_year:Int ,
  years_serv:Int,
  death_age:Maybe Int,
  party:String,
  state_born:String
 }

init : () -> (Model, Cmd Msg)
init _ =
    ({ presidents = Nothing }, getPresidents)


getPresidents : Cmd Msg
getPresidents =
  Http.get
    { url = "http://localhost:3000/president"
    , expect = Http.expectJson GotPresidents presidentsDecoder
    }


presidentsDecoder : Decoder (List President)
presidentsDecoder =
  list (
   (map6 President
    (field "pres_name" string)
    (field "birth_year" int)
    (field "years_serv" int)
    (field "death_age" (nullable int))
    (field "party" string)
    (field "state_born" string)
   )
  )

type Msg =
    GotPresidents (Result Http.Error (List President)) 


update : Msg -> Model ->  (Model, Cmd Msg)
update msg model =
    case msg of
        GotPresidents result ->
          case result of
              Ok l -> 
                ({ model | presidents = Just l }, Cmd.none)
              Err _ -> 
                ({ model | presidents = Nothing }, Cmd.none)


view : Model -> Html Msg
view model =
    div []
        (
        case model.presidents of
            Just l ->
              List.map (\p -> div [] [text p.pres_name])  l 
            Nothing ->
              []
        ) 
        


main =
  Browser.element
    { init = init
    , update = update
    , subscriptions = subscriptions
    , view = view
    }


subscriptions : Model -> Sub Msg
subscriptions model =
  Sub.none
